#! /usr/bin/perl
#---------------------------------------------------------------------

use Test::More tests => 1;

BEGIN {
    use_ok('Pod::Elemental::MakeSelector');
}

diag("Testing Pod::Elemental::MakeSelector $Pod::Elemental::MakeSelector::VERSION");
